@@ -178,6 +178,11 @@ mlt_producer producer_loader_init( mlt_profile profile, mlt_service_type type, c
 		mlt_properties_get( properties, "_xml" ) == NULL &&
 		mlt_properties_get( properties, "loader_normalised" ) == NULL )
 		attach_normalisers( profile, producer );
+	
+	// Always let the image and audio be converted
+	int created = 0;
+	create_filter( profile, producer, "imageconvert", &created );
+	create_filter( profile, producer, "audioconvert", &created );
 
 	// Now make sure we don't lose our identity
 	if ( properties != NULL )
